@@ -10,6 +10,10 @@ import datetime
 
 from lxml.html.clean import Cleaner
 
+
+# Function cache
+CACHE_SIZE = 8192
+
 # Download
 MAX_FILE_SIZE = 20000000
 MIN_FILE_SIZE = 10
@@ -24,7 +28,7 @@ LATEST_POSSIBLE = datetime.date.today()
 MAX_YEAR = LATEST_POSSIBLE.year
 
 # set an upper limit to the number of candidates
-MAX_POSSIBLE_CANDIDATES = 100
+MAX_POSSIBLE_CANDIDATES = 150
 
 # HTML_CLEANER config # http://lxml.de/api/lxml.html.clean.Cleaner-class.html
 HTML_CLEANER = Cleaner()
