@@ -14,16 +14,19 @@ import time
 from collections import Counter
 from functools import lru_cache
 
-from .settings import MIN_DATE, MIN_YEAR, LATEST_POSSIBLE, MAX_YEAR
+from .settings import CACHE_SIZE, LATEST_POSSIBLE, MAX_YEAR, MIN_DATE, MIN_YEAR
 
 
 LOGGER = logging.getLogger(__name__)
 LOGGER.debug('date settings: %s %s %s', MIN_YEAR, LATEST_POSSIBLE, MAX_YEAR)
 
 
-@lru_cache(maxsize=32)
+@lru_cache(maxsize=CACHE_SIZE)
 def date_validator(date_input, outputformat, earliest=MIN_DATE, latest=LATEST_POSSIBLE):
     """Validate a string w.r.t. the chosen outputformat and basic heuristics"""
+    # safety check
+    if date_input is None:
+        return False
     # try if date can be parsed using chosen outputformat
     if not isinstance(date_input, datetime.date):
         # speed-up
@@ -56,20 +59,21 @@ def date_validator(date_input, outputformat, earliest=MIN_DATE, latest=LATEST_PO
 def output_format_validator(outputformat):
     """Validate the output format in the settings"""
     # test in abstracto
-    if not isinstance(outputformat, str) or not '%' in outputformat:
+    if not isinstance(outputformat, str) or '%' not in outputformat:
         logging.error('malformed output format: %s', outputformat)
         return False
     # test with date object
     dateobject = datetime.datetime(2017, 9, 1, 0, 0)
     try:
         dateobject.strftime(outputformat)
-    except (NameError, TypeError, ValueError) as err:
+    # Python < 3.7 only
+    except (NameError, TypeError, UnicodeError, ValueError) as err:
         logging.error('wrong output format or format type: %s %s', outputformat, err)
         return False
     return True
 
 
-@lru_cache(maxsize=32)
+@lru_cache(maxsize=CACHE_SIZE)
 def plausible_year_filter(htmlstring, pattern, yearpat, tocomplete=False):
     """Filter the date patterns to find plausible years only"""
     # slow!
@@ -106,24 +110,23 @@ def plausible_year_filter(htmlstring, pattern, yearpat, tocomplete=False):
 def compare_values(reference, attempt, outputformat, original_date):
     """Compare the date expression to a reference"""
     timestamp = time.mktime(datetime.datetime.strptime(attempt, outputformat).timetuple())
-    if original_date is True:
-        if reference == 0 or timestamp < reference:
-            reference = timestamp
-    else:
-        if timestamp > reference:
-            reference = timestamp
+    if original_date is True and (reference == 0 or timestamp < reference):
+        reference = timestamp
+    elif original_date is False and timestamp > reference:
+        reference = timestamp
     return reference
 
 
-@lru_cache(maxsize=32)
+@lru_cache(maxsize=CACHE_SIZE)
 def filter_ymd_candidate(bestmatch, pattern, original_date, copyear, outputformat, min_date, max_date):
     """Filter free text candidates in the YMD format"""
     if bestmatch is not None:
         pagedate = '-'.join([bestmatch.group(1), bestmatch.group(2), bestmatch.group(3)])
-        if date_validator(pagedate, '%Y-%m-%d', earliest=min_date, latest=max_date) is True:
-            if copyear == 0 or int(bestmatch.group(1)) >= copyear:
-                LOGGER.debug('date found for pattern "%s": %s', pattern, pagedate)
-                return convert_date(pagedate, '%Y-%m-%d', outputformat)
+        if date_validator(
+            pagedate, '%Y-%m-%d', earliest=min_date, latest=max_date
+        ) is True and (copyear == 0 or int(bestmatch.group(1)) >= copyear):
+            LOGGER.debug('date found for pattern "%s": %s', pattern, pagedate)
+            return convert_date(pagedate, '%Y-%m-%d', outputformat)
             ## TODO: test and improve
             #if original_date is True:
             #    if copyear == 0 or int(bestmatch.group(1)) <= copyear:
